@@ -37,15 +37,19 @@ __all__ = [
     'is_dep_implied',
     'is_relation_implied',
     'parse_standards_version',
+    'PkgRelationFieldEditor',
     ]
 
 import collections
 import contextlib
 from itertools import takewhile
+import operator
 import os
+import re
 from typing import Optional, Callable, Tuple, Union, List, Iterable, Dict
 
 from debian.changelog import Version
+
 import subprocess
 import warnings
 
@@ -54,17 +58,72 @@ from .deb822 import (
     Deb822Editor,
     ChangeConflict,
     Deb822Paragraph,
-    new_deb822_paragraph,
     parse_deb822_paragraph,
     )
 from .reformatting import GeneratedFile
 
 
+# TODO(jelmer): dedupe with scripts/wrap-and-sort in devscripts
+CONTROL_LIST_FIELDS = (
+    "Breaks",
+    "Build-Conflicts",
+    "Build-Conflicts-Arch",
+    "Build-Conflicts-Indep",
+    "Build-Depends",
+    "Build-Depends-Arch",
+    "Build-Depends-Indep",
+    "Built-Using",
+    "Conflicts",
+    "Depends",
+    "Enhances",
+    "Pre-Depends",
+    "Provides",
+    "Recommends",
+    "Replaces",
+    "Suggests",
+    "Xb-Npp-MimeType",
+)
+
+
 def parse_relation(t: str):
     with warnings.catch_warnings():
         suppress_substvar_warnings()
         return PkgRelation.parse(t)
 
+# From scripts/wrap-and-sort in devscripts
+# Copyright (C) 2010-2018, Benjamin Drung <bdrung@debian.org>
+#               2010, Stefano Rivera <stefanor@ubuntu.com>
+
+
+def _sort_packages_key(package):
+    # Sort dependencies starting with a "real" package name before ones
+    # starting with a substvar
+    return 0 if re.match("[a-z0-9]", package) else 1, package
+
+
+def _wrap_field(control, entry, sort, formatter):
+    from debian._deb822_repro import LIST_COMMA_SEPARATED_INTERPRETATION
+    view = control.as_interpreted_dict_view(
+        LIST_COMMA_SEPARATED_INTERPRETATION)
+    with view[entry] as field_content:
+        seen = set()
+        for package_ref in field_content.iter_value_references():
+            value = package_ref.value
+            new_value = " | ".join(x.strip() for x in value.split("|"))
+            if not sort or new_value not in seen:
+                package_ref.value = new_value
+                seen.add(new_value)
+            else:
+                package_ref.remove()
+        if sort:
+            field_content.sort(key=_sort_packages_key)
+            if formatter:
+                field_content.value_formatter(formatter)
+            field_content.reformat_when_finished()
+
+
+# End import from devscripts
+
 
 def dh_gnome_clean(path: str = '.') -> None:
     """Run the dh_gnome_clean command.
@@ -300,7 +359,7 @@ class ControlEditor(object):
                 raise ValueError('first paragraph is not Source')
             return entry
         else:
-            p = new_deb822_paragraph()
+            p = Deb822Paragraph.new_empty_paragraph()
             self.paragraphs.insert(0, p)
             return p
 
@@ -391,14 +450,48 @@ class ControlEditor(object):
             self.changed = self._primary.changed
         return False
 
+    def sort_binary_packages(self, keep_first: bool = False) -> None:
+        sort_key = operator.itemgetter("Package")
+        self._primary.sort_paragraphs(
+            skip=1 + int(keep_first), sort_key=sort_key)
+
+    def wrap_and_sort(self, short_indent: bool = False,
+                      trailing_comma: bool = False,
+                      wrap_always: bool = False,
+                      max_line_length: int = 79) -> None:
+        """Wrap and sort this control file."""
+        try:
+            from devscripts.control import wrap_and_sort_formatter
+        except ImportError:
+            raise NotImplementedError('version of devscripts too old')
+        formatter = wrap_and_sort_formatter(
+            1 if short_indent else "FIELD_NAME_LENGTH",
+            trailing_separator=trailing_comma,
+            immediate_empty_line=short_indent,
+            max_line_length_one_liner=0 if wrap_always else max_line_length
+        )
+        for paragraph in self.paragraphs:
+            for field in CONTROL_LIST_FIELDS:
+                if field in paragraph:
+                    _wrap_field(paragraph, field, True, formatter)
+            if "Uploaders" in paragraph:
+                _wrap_field(paragraph, "Uploaders", False, formatter)
+            if "Architecture" in paragraph:
+                archs = list(paragraph["Architecture"].split())
+                # Sort, with wildcard entries (such as linux-any) first:
+                archs = sorted(archs, key=lambda x: ("any" not in x, x))
+                paragraph["Architecture"] = " ".join(archs)
+
     def add_binary(self, contents):
         if isinstance(contents, dict):
-            para = new_deb822_paragraph()
-            para.update(contents)
+            para = Deb822Paragraph.from_dict(contents)
         else:
             para = contents
         return self._primary.paragraphs.append(para)
 
+    def remove(self, para):
+        self._primary.paragraphs.remove(para)
+
 
 def parse_relations(text: str):
     """Parse a package relations string.
@@ -491,6 +584,10 @@ def iter_relations(
       Tuples with offset and relation objects
     """
     relations = parse_relations(relationstr)
+    return _iter_relations(relations, package)
+
+
+def _iter_relations(relations, package):
     for i, (head_whitespace, relation, tail_whitespace) in enumerate(
             relations):
         if isinstance(relation, str):  # formatting
@@ -544,7 +641,7 @@ def ensure_minimum_version(
             changed = True
     if not found:
         changed = True
-        _add_dependency(
+        _add_relation(
             relations,
             [PkgRelation(name=package, version=('>=', minimum_version))])
     for i in reversed(obsolete_relations):
@@ -588,7 +685,7 @@ def ensure_exact_version(
             changed = True
     if not found:
         changed = True
-        _add_dependency(
+        _add_relation(
             relations,
             [PkgRelation(name=package, version=('=', version))],
             position=position)
@@ -627,7 +724,7 @@ def ensure_relation(
                 relations[i] = (relations[i][0], new_relation, relations[i][2])
                 added = True
     if not added:
-        _add_dependency(relations, new_relation)
+        _add_relation(relations, new_relation)
 
     for i in reversed(to_remove):
         del relations[i]
@@ -635,11 +732,11 @@ def ensure_relation(
     return format_relations(relations)
 
 
-def _add_dependency(
+def _add_relation(
         relations: List[Tuple[str, List[PkgRelation], str]],
         relation: List[PkgRelation],
         position: Optional[int] = None) -> None:
-    """Add a dependency to a depends line.
+    """Add a relation to a list of relations.
 
     Args:
       relations: existing list of relations
@@ -712,7 +809,7 @@ def add_dependency(relationstr, relation, position=None):
     relations = parse_relations(relationstr)
     if isinstance(relation, str):
         relation = parse_relation(relation)
-    _add_dependency(relations, relation, position=position)
+    _add_relation(relations, relation, position=position)
     return format_relations(relations)
 
 
@@ -737,7 +834,7 @@ def ensure_some_version(relationstr: str, package: str) -> str:
         if names != [package]:
             continue
         return relationstr
-    _add_dependency(relations, parse_relation(package))
+    _add_relation(relations, parse_relation(package))
     return format_relations(relations)
 
 
@@ -918,3 +1015,109 @@ def suppress_substvar_warnings():
         category=UserWarning,
         message=(r'cannot parse package relationship \"\$\{.*\}\", returning '
                  r'it raw'))
+
+
+class PkgRelationFieldEditor(object):
+    """Convenience wrapper for editing pkg relation fields."""
+
+    def __init__(self, paragraph, name):
+        self.paragraph = paragraph
+        self.name = name
+        self._parsed = None
+
+    def __enter__(self):
+        v = self.paragraph.get(self.name)
+        if v is not None:
+            self._parsed = parse_relations(v)
+        else:
+            self._parsed = None
+
+        return self
+
+    def drop_relation(self, package):
+        """Drop a relation.
+
+        Args:
+          package: package name
+        """
+        if self.value is None:
+            return
+
+        def keep(relation):
+            names = [r.name for r in relation]
+            return set(names) != set([package])
+        new_parsed = filter_dependencies(self._parsed, keep)
+        ret = self._parsed != new_parsed
+        self._parsed = new_parsed
+        return ret
+
+    def __bool__(self):
+        return self._parsed is not None
+
+    def add_relation(self, relation, position=None):
+        """Add a relation.
+
+        Args:
+          relation: New relation
+          position: Optional position to insert relation at (defaults to last)
+        """
+        if isinstance(relation, str):
+            relation = parse_relation(relation)
+        return _add_relation(self._parsed, relation, position=position)
+
+    def iter_relations(self, package):
+        """Iterate over all relations with a particular package.
+
+        Args:
+          package: Package name
+        Yields:
+          Tuples with offset and relation objects
+        """
+        if self._parsed is None:
+            return
+        return _iter_relations(self._parsed, package)
+
+    def get_relation(self, package):
+        """Retrieve the relation for a particular package.
+
+        Args:
+          relationstr: package relation string
+          package: package name
+        Raises:
+          ValueError if there is more than one relation with the package
+        Returns:
+          Tuple with offset and relation object
+        """
+        for offset, relation in self.iter_relations(package):
+            names = [r.name for r in relation]
+            if len(names) > 1 and package in names:
+                raise ValueError("Complex rule for %s , aborting" % package)
+            if names != [package]:
+                continue
+            return offset, relation
+        raise KeyError(package)
+
+    def has_relation(self, package):
+        """Check whether there is a relation with specified package.
+
+        Args:
+          package: name of the package
+        Returns: boolean
+        """
+        try:
+            self.get_relation(package)
+        except KeyError:
+            return False
+        else:
+            return True
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        if exc_type is None:
+            if self._parsed is None:
+                try:
+                    del self.paragraph[self.name]
+                except KeyError:
+                    pass
+            else:
+                self.paragraph[self.name] = format_relations(self._parsed)
+        return False
