@@ -23,63 +23,34 @@ __all__ = [
     'reformat_deb822',
     'ChangeConflict',
     'Deb822Editor',
-    'new_deb822_paragraph',
     'parse_deb822_file',
     'parse_deb822_paragraph',
     'Deb822Paragraph',
-    ]
+]
 
 from io import BytesIO
-from typing import Iterable, List, Optional
-
-from debian.changelog import Version
-
-try:
-    from debian import __version__ as python_debian_version
-    parsed_python_debian_version = Version(python_debian_version)
-    # Disabled until bug 996785 is fixed.
-    raise ModuleNotFoundError
-    if parsed_python_debian_version < Version('0.1.42+git20211123'):
-        raise ModuleNotFoundError
-    try:
-        from debian._deb822_repro.parsing import (
-            parse_deb822_file,
-            Deb822ParagraphElement as Deb822Paragraph,
-            Deb822FileElement as Deb822File,
-            )
-    except TypeError:
-        # This happens on python 3.7 with older versions of python3-debian
-        raise ModuleNotFoundError
-    if parse_deb822_file is None:
-        # On python < 3.9, this is the case
-        raise ModuleNotFoundError
-
-    def parse_deb822_paragraph(p):
-        f = parse_deb822_file(p)
-        [p] = f.iter_parts_of_type(Deb822Paragraph)
-        return p
-
-    def new_deb822_paragraph():
-        return Deb822Paragraph.new_empty_paragraph()
-
-    has_deb822_repro = True
-except ModuleNotFoundError:
-    from debian.deb822 import Deb822
-
-    def parse_deb822_file(c):  # type: ignore
-        return list(Deb822.iter_paragraphs(c))
-
-    Deb822Paragraph = Deb822  # type: ignore
-    Deb822File = Iterable[Deb822Paragraph]  # type: ignore
-    parse_deb822_paragraph = Deb822
-    new_deb822_paragraph = Deb822
-
-    has_deb822_repro = False
+from typing import List, Optional
 
+from debian._deb822_repro.parsing import (
+    parse_deb822_file,
+    Deb822ParagraphElement as Deb822Paragraph,
+    Deb822FileElement as Deb822File,
+)
 
 from .reformatting import (
     Editor,
-    )
+)
+
+
+# Urgh; this is bad form as a library, but the alternative is
+# silently discarding comments.
+Deb822Paragraph._discard_comments_on_read = False
+
+
+def parse_deb822_paragraph(p):
+    f = parse_deb822_file(p)
+    [p] = f.iter_parts_of_type(Deb822Paragraph)
+    return p
 
 
 def dump_paragraphs(paragraphs: Deb822File) -> bytes:
@@ -185,7 +156,7 @@ class Deb822Editor(Editor):
                         paragraph[key] = new_value
         # Add any new paragraphs that weren't processed earlier
         for key, p in changes.items():
-            paragraph = new_deb822_paragraph()
+            paragraph = Deb822Paragraph.new_empty_paragraph()
             for (field, old_value, new_value) in p:
                 if old_value is not None:
                     new_value = resolve_conflict(
@@ -209,3 +180,12 @@ class Deb822Editor(Editor):
         if self.allow_missing:
             return parse_deb822_file([])
         raise
+
+    def sort_paragraphs(self, sort_key, skip=0):
+        sortable = list(self.paragraphs)[skip:]
+        for i, p in reversed(list(enumerate(self.paragraphs))):
+            if i < skip:
+                continue
+            self.paragraphs.remove(p)
+        for p in sorted(sortable, key=sort_key):
+            self.paragraphs.append(p)
