@@ -18,8 +18,8 @@
 """Utility functions for dealing with changelog files."""
 
 __all__ = [
-    'get_vendor',
-    ]
+    'get_vendor_name',
+]
 
 
 import os
